/*
 ** Copyright (C) 2002-2010 Sourcefire, Inc.
 ** Copyright (C) 1998-2002 Martin Roesch <roesch@sourcefire.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
 ** published by the Free Software Foundation.  You may not use, modify or
 ** distribute this program under any other version of the GNU General
 ** Public License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ** 
 ** Description
 ** This file contains the utility functions used by rule options.
 **
 */

#include "decode.h"


extern const u_char *file_data_ptr;
extern uint8_t base64_decode_buf[DECODE_BLEN];
extern uint32_t base64_decode_size;
extern uint32_t mime_decode_size;
extern uint8_t mime_present;
extern uint8_t doe_buf_flags;
extern const uint8_t *doe_ptr;

#define DOE_BUF_URI     0x01
#define DOE_BUF_STD     0x02
/*
 * Function: setFileDataPtr
 *
 * Purpose: Sets the file data pointer used by 
 *          file_data rule option.
 *
 * Arguments: ptr => pointer to the body data
 *
 * Returns: void
 *
 */

static INLINE void setFileDataPtr(const u_char *ptr, uint32_t decode_size)
{
    file_data_ptr = ptr;
    mime_decode_size = decode_size;

}

/*
 * Function: IsBase64DecodeBuf
 *
 * Purpose: Checks if there is base64 decoded buffer.
 *
 * Arguments: p => doe_ptr
 *
 * Returns: Returns 1 if there is base64 decoded data
 *          and if the doe_ptr is within the buffer.
 *          Returns 0 otherwise.
 *
 */

static INLINE int IsBase64DecodeBuf(const uint8_t *p)
{
    if( base64_decode_size && p )
    {
        if ((p >= base64_decode_buf) && 
                (p < (base64_decode_buf + base64_decode_size)))
        {
            return 1;
        }
        else
            return 0;
    }
    else
        return 0;
}

static INLINE int IsMimeDecodeBuf(const uint8_t *p)
{
    if( mime_present && file_data_ptr)
    {
        if ((p >= file_data_ptr) &&
                (p < (file_data_ptr + mime_decode_size)))
        {
            return 1;
        }
        else
            return 0;
    }
    else
        return 0;
}

/*
 * Function: SetDoePtr(const uint8_t *ptr, uint8_t type)
 *
 * Purpose: This function set the doe_ptr and sets the type of
 *          buffer to which doe_ptr points.
 * 
 * Arguments: ptr       => pointer
 *            type      => type of buffer
 * 
 * Returns: void
 *
*/

static INLINE void SetDoePtr(const uint8_t *ptr, uint8_t type)
{
    doe_ptr = ptr;
    doe_buf_flags = type;
}

/*
 * Function: UpdateDoePtr(const uint8_t *ptr, uint8_t update)
 *
 * Purpose: This function updates the doe_ptr and resets the type of
 *          buffer to which doe_ptr points based on the update value.
 * 
 * Arguments: ptr       => pointer
 *            update    => reset the buf flag if update is not zero.
 * 
 * Returns: void
 *
*/

static INLINE void UpdateDoePtr(const uint8_t *ptr, uint8_t update)
{
    doe_ptr = ptr;
    if(update)
        doe_buf_flags = DOE_BUF_STD;
}
